package net.h34t.filemure.repository

import net.h34t.filemure.LimboFile
import net.h34t.filemure.generateExtId
import java.io.InputStream
import java.sql.Connection
import java.sql.DriverManager
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

class SqliteRepository(url: String) {

    private val dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")

    val connection: Connection = DriverManager.getConnection(url)

    fun addFileToLimbo(accountId: Long, filename: String, contentType: String?, size: Long, content: InputStream) {
        connection.prepareStatement("INSERT INTO file (account_id, ext_id, filename, content_type, file_size, content) VALUES (?,?,?,?,?,?)")
            .use { stmt ->
                stmt.setLong(1, accountId)
                stmt.setString(2, generateExtId())
                stmt.setString(3, filename)
                stmt.setString(4, contentType)
                stmt.setLong(5, size)
                stmt.setBytes(6, content.readAllBytes())

                val res = stmt.executeUpdate()

                require(res == 1)
            }
    }

    fun getLimboFileCount(accountId: Long): Long {
        connection.prepareStatement("SELECT count(*) AS count FROM file WHERE account_id=? AND document_id IS NULL")
            .use { stmt ->
                stmt.setLong(1, accountId)
                val rs = stmt.executeQuery()
                rs.next()
                return rs.getLong(1)
            }
    }

    fun getFilesInLimbo(accountId: Long): List<LimboFile> {
        connection.prepareStatement("SELECT id, account_id, filename, content_type, file_size, created FROM file WHERE account_id=? AND document_id IS NULL ORDER BY created DESC")
            .use { stmt ->
                stmt.setLong(1, accountId)
                val res = stmt.executeQuery()

                val list = mutableListOf<LimboFile>()

                while (res.next()) {
                    list += LimboFile(
                        id = res.getLong(1),
                        accountId = res.getLong(2),
                        filename = res.getString(3),
                        contentType = res.getString(4),
                        size = res.getLong(5),
                        created = LocalDateTime.parse(res.getString(6), dtf)
                    )
                }

                return list
            }
    }

    fun addDocument(
        accountId: Long,
        title: String,
        referenceDate: LocalDateTime,
        tags: List<String>,
        description: String,
        fileIds: List<Long>
    ): Long {
        val savePoint = connection.setSavepoint()
        try {
            val documentId = connection.prepareStatement(
                """INSERT INTO document 
            |(account_id, ext_id, title, description, tags, created, reference_date) 
            |VALUES 
            |(?, ?, ?, ?, ?, datetime(), ?)""".trimMargin()
            ).use { stmt ->
                stmt.setLong(1, accountId)
                stmt.setString(2, generateExtId())
                stmt.setString(3, title)
                stmt.setString(4, description)
                stmt.setString(5, tags.joinToString(","))
                stmt.setString(6, referenceDate.format(dtf))
                stmt.executeUpdate()
                val gks = stmt.generatedKeys
                gks.next()
                gks.getLong(1)
            }

            val ids = fileIds.joinToString(",")

            connection.prepareStatement("""UPDATE file SET document_id=? WHERE account_id=? AND id IN ($ids)""")
                .use { stmt ->
                    stmt.setLong(1, documentId)
                    stmt.setLong(2, accountId)
                    val affected = stmt.executeUpdate()
                    require(affected == fileIds.size)
                }

            connection.commit()
            return documentId
        } catch (exception: Exception) {
            connection.rollback(savePoint)
            throw exception
        }
    }
}